<?xml version="1.0" encoding="UTF-8"?>
<gui name="ksudoku"
     version="7"
     xmlns="http://www.kde.org/standards/kxmlgui/1.0"
     xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
     xsi:schemaLocation="http://www.kde.org/standards/kxmlgui/1.0
                         http://www.kde.org/standards/kxmlgui/1.0/kxmlgui.xsd">

<MenuBar>
	<Menu noMerge="1" name="game" ><text>&amp;Game</text>
		<Action group="game_merge" name="game_new"  />
		<Action group="game_merge" name="game_restart" />
		<Action group="game_merge" name="game_load" />
		<Separator group="game_merge" />
		<Action group="save_merge" name="game_save" />
		<Action group="save_merge" name="game_save_as" />
		<Separator group="save_merge" />
		<Action group="print_merge" name="game_print" />
		<Separator group="print_merge" />
		<Action group="game_merge" name="game_quit" />
	</Menu>

	<Menu noMerge="1" name="move"><text>&amp;Move</text>
		<Action name="move_undo" group="move_undo_merge" />
		<Action name="move_redo" group="move_undo_merge" />
		<Action name="move_add_group" group="move_undo_merge"/>
		<Action name="move_undo_group" group="move_undo_merge"/>
		<Separator group="move_undo_merge" />
		<Action name="move_hint" />
		<Action name="move_solve" />
		<Action name="move_dub_puzzle" />
	</Menu>

	<Menu name="settings">
		<Action name="enable_messages"/>
	</Menu>

	<Menu name="help"><text>&amp;Help</text>
		<Action name="Home_page" />
		<Action name="support" />
		<Action name="sendComment" />
	</Menu>
</MenuBar>

<ToolBar name="mainToolBar">
	<text>Main Toolbar</text>
	<Action name="game_new" />
	<Action name="game_restart" />
	<Action name="game_print" />
</ToolBar>

<ToolBar noMerge="1" name="actionsToolbar" position="top">
	<text>Actions Toolbar</text>
	<Action name="move_hint" />
	<Action name="move_solve" />
	<Action name="move_dub_puzzle" />
</ToolBar>

<State name="dubbing">
	<enable>
		<Action name="move_dub_puzzle" />
	</enable>
	<disable>
		<Action name="move_hint"/>
		<Action name="move_solve"/>
	</disable>
</State>

</gui>
